#ifndef ACOMP_IUPUI_INCLUDED
#define ACOMP_IUPUI_INCLUDED

#include <string.h>
#include <math.h>
#include <stdio.h> 
#include <stdlib.h>
#include "allocate.h"

#define CLIP_NRM_A

/*struct A_correction{
  double **Y_t;
  char mode;  
  };*/

struct column_flt{
 float *val;
 int *row;
 int n_row;
 double multiplier;
  /*struct A_correction A_cor;*/
};


struct sysmat_private{
  double *cs_rot,*si_rot;
  struct column_flt **sp_col_arr;
  double r_sq;

};

struct deadtime_info{
  int *scan_start_second; /* [sweep-file] */
  int *scan_start_minute; /* [sweep-file] */
  int *scan_start_hour;   /* [sweep-file] */
  int *scan_start_day;   /* [sweep-file] */
  double ***cor_singles;  /* [sweep-file][frame][detector_block] */
  double ***uncor_singles;
  int no_sino_files;
};


struct attenuation_info
{
  double *attn_arr;      /* final attenuation sino-1D-array */

  int ATTN_MODE;         /* 0: off, 1: file, 2: circle */  
  double **sin_img;      /* sin-file matrix[ind_theta][ind_t] */
  int img_rows,img_cols;

  double attn_analyt_r;  /* radius of circle */
  double attn_cnst;      /* attn_constant in unit invcm */
};


struct sino_info{
  int    n_theta;
  int    n_t;
  double  delta_theta;
  double  delta_t;
  double  theta_0;
  double  t_0;
  int dec_a, dec_t;
  double sep_d;
  int no_frames;
  double *theta_arr;
  double *t_arr;
  float *nrm_arr;
  double **nrm_img;
  float *decay_arr;
  int *mask_arr;
  int nrm_FLAG; /* 0= no det. normalization, 1=det normalization */
  double halflife;
  int **frames_starttime;
  int **frames_duration;


  int DECAY_FLAG;
  /* vars below still need to be set in prg */
  int frame_start_time;
  int frame_duration;
  int plane;
  double dthck;
  
  double prj_max_t;/* max displacement projections used in the reconstruction */ 
  short **pix_reconst_mask;
  
  int analyt_sysresp;
  
  int no_planes; 
  
  struct sysmat_private sys_priv;

  double *dtimecr_arr;
  int DEADTIME_FLAG;

  struct deadtime_info dtime;
  struct attenuation_info attn;

  double *sino_cor_arr; /* contains multiplication dtime*attn corrections */
  int SINO_COR_FLAG;

};





struct column{
 double *val;
 int *row;
 int n_row;
 double multiplier;
  /*struct A_correction A_cor;*/
};





struct geom_info{
 double delta;
 double x_0;
 double y_0;
 int n_y;
 int n_x;
 double dia;
 double field_of_view;
 };

struct resp_matrix{
  int N_t;
  int N_th;
  double th_range;
  double t_range;
  float **data;
  double x_pos;
  double y_pos;

};



struct system_resp{
  int analytical;
  int no_matrices;
  double ddet;
  double dthck;
  double rdet;
  double sep_d;
  
  struct resp_matrix *resmat;
};

void A_comp(
 int im_row, int im_col,
 struct sino_info *sino,
 struct geom_info *geom,
 double **pix_prof,
 struct column *sp_col, int res_level);

void init_A_matrix(struct sino_info *sino,struct geom_info *geom, char *fname_sysres);
void free_A_matrix(struct sino_info *sino, struct geom_info *geom);
void calc_sino_arrays(struct sino_info *sino,struct system_resp *sysres);
void read_system_response(char *fname, struct system_resp *sysres);
void checkstat(int stat,int num);
int** calculate_frame_times(struct sino_info *sino, int no_frames);
void calculate_deadtime_correction(struct sino_info *sino, int no_frames);
void calc_deadtime_blocks(int **det1, int **det2);
void calculate_attenuation_correction(struct sino_info *sino);
void correct_sino_forcbp(struct sino_info *sino,double **Y);
void resort_sino_forcbp(double **Ycbp,struct sino_info *sino,double **Y, int nang);


#ifdef MATLAB5
 #include <mat.h>

  void read_mat_sino_info(
    MATFile *fin,       /* pointer to file */
    struct sino_info *sino /* Structure of specs from sinogram    */
    );

#endif
#endif
