#ifndef INC_CTI_STRUCTS
#define INC_CTI_STRUCTS

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "allocate.h"
#include "cti_byte_io.h"

struct main_head{			/* bytes already read */
 short fill[16];			/* 28 */
 char original_file_name[22];		/* 48 */
 short sw_version;			/* 50 */
 short data_type;			/* 52 */
 short system_type;			/* 54 */
 short file_type;			/* 56 */
 char node_id[12];			/* 66 */
 short scan_start_day;			/* 68 */
 short scan_start_month;
 short scan_start_year;
 short scan_start_hour;
 short scan_start_minute;
 short scan_start_second;		/* 78 */
 char isotope_code[10];			/* 86 */
 float isotope_halflife;		/* 90 */
 char radiopharmaceutical[34];		/* 122 */
 float gantry_tilt;
 float gantry_rotation;
 float bed_elevation;			/* 134 */
 short rot_source_speed;		/* 136 */
 short wobble_speed;
 short transm_source_type;		/* 140 */
 float axial_fov;		
 float transaxial_fov;			/* 148 */
 short transaxial_samp_mode;
 short coin_samp_mode;
 short axial_samp_mode;			/* 154 */
 float calibration_factor;		/* 158 */
 short calibration_units;
 short compression_code;		/* 162 */
 char study_info[190];			/* 350 */
 short aquisition_type;
 short bed_type;
 short septa_type;			/* 356 */
 char facility_name[22];		/* 376 */
 short num_planes;
 short num_frames;
 short num_gates;
 short num_bed_pos;			/* 384 */
 float init_bed_pos;			/* 388 */
 float bed_offsets[17];			/* 448 */
 float plane_separation;		/* 452 */
 short lwr_sctr_thres;
 short lwr_true_thres;
 short upr_true_thres;			/* 458 */
 float collimator;			/* 462 */
 short fill2[27];			/* 512 */
};


struct scn_head{
  short fill[65];                       /* read 63 shts, 126 */
  short data_type;                      /* 128 */
  short fill1[4];                       /* read 2 shts, 132 */
  short dimension_1;
  short dimension_2;
  short smoothing;
  short processing_code;                /* 140 */
  short fill2[5];                       /* read 3 shts, 146 */
  float sample_distance;                /* 150 */
  short fill3[10];                      /* read 8 shts, 166 */
  float isotope_halflife;               /* 170 */
  short frame_duration_sec;             /* 172 */
  long gate_duration;                   /* 176 */
  long r_wave_offset;                   /* 180 */
  short fill4;                          /* 182 */
  float scale_factor;                   /* 186 */
  short fill5[5];                       /* read 3 sht, 192*/
  short scan_min;                       /* 194 */
  short scan_max;                       /* 196 */
  long prompts;                         /* 200 */
  long delayed;                         /* 204 */
  long multiples;                       /* 208 */
  long net_trues;                       /* 212 */
  short fill6[54];                      /* read 52 shts, 316 */
  float cor_singles[18];                /* read 16 flts, 380 */
  float uncor_singles[18];              /* read 16 flts, 444 */
  float tot_avg_cor;                    /* 448 */
  float tot_avg_uncor;                  /* 452 */
  long tot_coin_rate;                   /* 456 */
  long frame_start_time;                /* 460 */
  long frame_duration;                  /* 464 */
  float loss_correction_factor;         /* 468 */
  long phy_planes[10];                  /* read 8 longs, 500 */
  short fill7[8];                       /* read 6 shorts, 512 */
};

struct img_shead{
  short fill[65];  /* 126   : write 63 shorts */
  short data_type; /* 128 */        /* all below + 2*/
  short num_dimensions; /* 128 */
  short fill2;          /* 130 */
  short dimension_1;    /* 132 */
  short dimension_2;    /* 134 */
  short fill3[14];      /* 158 */ 
  float x_origin;       /* 162 */
  float y_origin;       /* 166 */
  float recon_scale;    /* 170 */
  float quant_scale;    /* 174 */
  short image_min;      /* 176 */
  short image_max;      /* 178 */
  short fill4[4];       /* 182 write 2 */
  float pixel_size;     /* 186 */
  float slice_width;    /* 190 */
  long frame_duration;  /* 194 */
  long frame_start_time;/* 198 */
  short slice_location; /* 200 */
  short recon_start_hour;
  short recon_start_min;
  short recon_start_sec;
  long recon_duration;  /* 210 */
  short fill5[14];      /* 234 */
  short filter_code;    /* 236 */
  long scan_matrix_num; 
  long norm_matrix_num;
  long atten_cor_mat_num; /* 248*/
  short fill6[25];        /* 294 */
  float image_rotation;   /* 298 */
  float plane_eff_corr_fctr; /* 302 */
  float decay_corr_fctr;  /* 306 */
  float loss_corr_fctr;   /* 310 */
  short fill7[34];        /* 374 */
  short processing_code;  /* 376 */
  short fill8;
  short quant_units;      /* 380 */
  short recon_start_day;
  short recon_start_month;
  short recon_start_year; /* 386 */
  float ecat_calibration_fctr; /* 390 */
  float well_counter_cal_fctr; /* 394 */
  float filter_params[8]; /* 418 */
  char annotation[42]; /* 458 */
  short fill9[28]; /* 510 */
};


void read_main_header(struct main_head *mh, FILE *fp, int BYTE_ORDER);
void write_main_header(FILE *fp, struct main_head *mh, int BYTE_ORDER);
void init_main_header(struct main_head *mh);
void print_main_head(struct main_head *mh);

void read_subheader(struct scn_head *sh,FILE *fp, int BYTE_ORDER);
void write_subheader(struct scn_head *sh,FILE *fp, int BYTE_ORDER);
void init_subheader(struct scn_head *sh);
void print_subheader(struct scn_head *sh);

void init_img_head(struct img_shead *ish);
void write_img_shead(FILE *fp, struct img_shead *sh, int BYTE_ORDER);




#endif
