#include <stdio.h>
#include <string.h>
#include <math.h>


#include "defs.h"
#include "read_cfg_file.h"
#include "iu_utils.h"

#include "ansi_lib_loc/A_comp_iupui.h"
#include "ansi_lib_loc/allocate.h"
#include "ansi_lib_loc/cti_io.h"
#include "ansi_lib_loc/io_mat_dbl.h"
#include "ansi_lib_loc/fft.h"





#ifdef __linux__
#include <getopt.h>
#endif
extern int optind;
extern char *optarg;
int VERBOSE=1;

#define PI_1        3.14159265358979323846

/*#define WRITE_FILTER*/

void change_parameters_for_sgl_frame_recon(struct all_data *data, struct cmdline_params *cmdline);
void cmdline_error(void);
void parse_cmdline(struct cmdline_params *cmdline,int argc,char **argv);
void print_configuration(struct cfg_info *cfg_info, struct cmdline_params *cmdline, struct all_data *data);
void set_parameters(struct cmdline_params *cmdline,struct cfg_info *cfg_info,struct all_data *data,struct geom_info *geom);


void read_data(struct all_data *data,struct column *sp_col,struct cfg_info *cfg_info,int resol);
void write_output(struct cfg_info *cfg_info,struct all_data *data,double sigma,struct geom_info *geom);


void init_sp_col(struct column *sp_col,struct sino_info *sino);
double recon(struct all_data *data,struct geom_info *geom,struct column *sp_col,struct cfg_info *cfg_info);


void free_memory(struct all_data *data, struct column *sp_col,struct cfg_info *cfg_info,double **Ycbp);


double calc_pmass(double **Y, struct sino_info *sino, struct geom_info *geom);

double snc(double n);
void cut_freq(double *filter,double del,int n);
void mean_value_correct(double **image, double pmass, struct geom_info *geom);
void filter_sino(double **Y,struct sino_info *sino,struct cmdline_params *cmdline);
double* GHamming_Filter(int n,struct cmdline_params *cmdline,struct sino_info *sino);
double* ramp_Filter(int n,struct cmdline_params *cmdline,struct sino_info *sino);
void conv(double *out, double *in1, int len1, double *in2, int len2);
void normalize_filter_to_sum1(double *filt, int len);
double hrl(int m, struct sino_info *sino);
double dhrl(double m, struct sino_info *sino);

/* routines for backproj */
void backproject(double **image,double **Y,struct sino_info *sino,struct geom_info *geom);
double interpolate(double findex, double **Y, int angle_index);
double min1(double **img,int N, int M);
double max1(double **img,int N, int M);







void main( int argc, char **argv )
{
  struct geom_info geom;
  struct sino_info sino_cbp;
  struct column sp_col;
  struct cmdline_params cmdline;
  struct all_data data;
  double sigma=0;
  struct cfg_info cfg_info;
  struct system_resp sysres;
  double **Ycbp;
  int i,j;
  /*FILE *fp;*/
  double pmass;






  parse_cmdline(&cmdline,argc,argv);

  printf("Reading configuration file %s...",cmdline.cfg_fname);fflush(stdout);
  read_cfg_file(cmdline.cfg_fname,&cfg_info);
  printf("done.\n\n");fflush(stderr);fflush(stdout);

  set_parameters(&cmdline,&cfg_info,&data,&geom);
  check_parameters(&cfg_info);

  printf("Reading files...\n");fflush(stdout);
  read_data(&data,&sp_col,&cfg_info,cfg_info.resol);
  printf("done reading files.\n\n");fflush(stderr);fflush(stdout);

  /* If single frame reconstruction specified in cmdline,
     set no_frames=1, copy frame sino data to the beg. of Y etc. */
  if (cmdline.single_frame!=-1)
    change_parameters_for_sgl_frame_recon(&data,&cmdline);
      
  if (cfg_info.verbose)
    print_cfg_info(&cfg_info);

  /*if (cfg_info.addposi_FLAG)
    y_positivity(&data);*/
  
  subtract_scatter_clip_y(&data);

  
  /* set sinogram entries outside diamond to zero */
  y_mask(&data);

  /* Read system response file , needed for scanner parameters*/
  read_system_response(cfg_info.sysres_fname,&sysres);
 
  if (fabs(sysres.sep_d-data.sino.sep_d)>1e-2)
    {
      fflush(stdout);
      fflush(stderr);
      fprintf(stderr,"Error: sysres.sep_d=%f != data.sino.sep_d=%f\n\a\n",sysres.sep_d,data.sino.sep_d);
      fflush(stderr);
      exit(-1);
    }

  calc_sino_arrays(&data.sino,&sysres);
    
  if (cfg_info.verbose)
    print_configuration(&cfg_info, &cmdline, &data);
  

  

  correct_sino_forcbp(&data.sino,data.Y);
  

  /* sorts into a 180 angles * 111 displacement sinogram */
  Ycbp=(double**)multialloc(sizeof(double),2,180,111);
  resort_sino_forcbp(Ycbp,&data.sino,data.Y,32);
  
  /*
    fp=fopen("resort","w");
    if (fp==NULL)
    {
    fprintf(stderr,"Can't write resort\n\a\n");
    exit(-1);
    }
    
    for (i=0;i<180;i++)
    {
    for (j=0;j<111;j++)
    fprintf(fp,"%g ",Ycbp[i][j]);
    fprintf(fp,"\n");
    }
    fclose(fp);
  */


  /* Set sino_cbp struct */
  sino_cbp.n_t=111;
  sino_cbp.n_theta=180;
  sino_cbp.delta_t=0.165;
  sino_cbp.delta_theta=PI_1/(double)sino_cbp.n_theta;
  sino_cbp.t_0=-((double)sino_cbp.n_t - 1.0)/2.0 * sino_cbp.delta_t;
  sino_cbp.theta_0 = -PI_1;
 

  /* Calculate pmass of input sinogram for mean */
  /* value normalization later                  */
  pmass=calc_pmass(Ycbp, &sino_cbp, &geom);
  
  /* allocate and init output image */
  data.X=(double**)get_img(geom.n_x,geom.n_y, sizeof(double));
  for(i=0; i<geom.n_y; i++)
    for(j=0; j<geom.n_x; j++) 
      data.X[i][j]=0.0;
  
  /* Filter Sinogram */
  filter_sino(Ycbp,&sino_cbp,&cmdline);

  /* Backprojection */
  backproject(data.X,Ycbp,&sino_cbp,&geom);

  /* mean-value correction */
  mean_value_correct(data.X, pmass, &geom);

  
  /* Write Matlab output */
  write_output(&cfg_info,&data,sigma,&geom);
  

  
  /*free_A_matrix(&data.sino,&geom);*/
  free_memory(&data, &sp_col,&cfg_info,Ycbp);

}



void filter_sino(double **Y,struct sino_info *sino,struct cmdline_params *cmdline)
{
  double *data,*filter,*ans;
  int n,i,j;

#ifdef WRITE_FILTER
  FILE *fp;
#endif

  printf("Filtering...\n");

  /* Calculate filter-length */
  n = (int) ceil(log(sino->n_t)/log(2.0));
  n = pow(2,n);

  /* Calculate Filter  of length (2*n) */
  if (cmdline->use_ramp_filter)
    filter=ramp_Filter(n,cmdline,sino);
  else
    filter=GHamming_Filter(n,cmdline,sino);

  /* Allocate space for data */
  ans = (double *)get_spc(4*n, sizeof(double));
  data = (double *)get_spc(2*n, sizeof(double));

  /* Pad data with zeros */
  for(i=sino->n_t;i<2*n;i++)
    data[i] = 0.0;
 

  /* Filter sinogram Y for each projection angle */
  for(i=0;i<sino->n_theta;i++) 
    {
      for(j=0;j<sino->n_t;j++)
	data[j] = (double)Y[i][j];
  
      convlv(data-1,2*n,filter-1,ans-1);
      
      /* insert into Y */
      for(j=0;j<sino->n_t;j++)
	Y[i][j] = (double)ans[j]; 
      
    }



#ifdef WRITE_FILTER
  /* Write filter response to disk for checking */
  fp=fopen("filter","wa");
  for (i=0;i<2*n;i++)
    fprintf(fp,"%g ",filter[i]);
  fclose(fp);
#endif
  
  free((void *)(ans));
  free((void *)(data));
  free((void *)(filter));
} 


double* GHamming_Filter(int n,struct cmdline_params *cmdline,struct sino_info *sino)
{
  FILE *fp;
  double *filter;
  int i;
  char fname[512];
  
  if (n!=128)
    {
      fprintf(stderr,"Current filter implementation for 256 only\n\a\n");
      exit(-1);
    }
  filter = (double *)get_spc(2*n, sizeof(double));
  
  sprintf(fname,"filters/gen_ham_%1.1f",cmdline->freq_cutoff);
  fp=fopen(fname,"r");
  if (fp==NULL)
    {
      fprintf(stderr,"Error: Can't open filter file %s.\n\a\n",fname);
      exit(-1);
    }
  printf("Reading filter file: %s\n",fname);
  for (i=0;i<2*n;i++)
    fscanf(fp,"%lg",&filter[i]);
  fclose(fp);

  for (i=0;i<2*n;i++)
    filter[i] = filter[i]/sino->delta_t;


  return(filter);

}



/* This reconstruction filter GHamming_Filter has frequency response

      H(omega) = H_ramp(omega) * (0.5 + 0.5cos(pi*omega/omega_c))

   where H_ramp is the ideal ramp reconstruction filter, 
   and omega_c is the cutoff frequency. 
   The filter input parameter specifies the ratio omega/omega_c .
*/
double* GHamming_Filter_old(int n,struct cmdline_params *cmdline,struct sino_info *sino)
{
  double *filter;
  int i;
  double del;


  /* allocate filter */
  filter = (double *)get_spc(2*n, sizeof(double));
  
  del = cmdline->freq_cutoff;
  printf("cos delay=%f, so cutoff is %f percent.\n",del,100.0*del); 
  
  /* Calculate impulse response in time domain, 
     hrl is ideal ramp filter response 1/(4*delta_t)*(2*delta(n)-sinc^2(n/2)),
     dhrl is basically same as hrl but w/ double arithm. input, 
          and uses 1/(4*delta_t)*(2*sinc(n)-sinc^2(n/2))
  */
  for(i=-n+1;i<=n;i++)
    filter[i+n-1] = 0.5*(hrl(i,sino) + 0.5*(dhrl(i-1.0/del,sino) + dhrl(i+1.0/del,sino) )); 
  
  /* So far, filter response is only accurate for omega<=omega_c */
  /* convolve with sinc() to chop off frequency domain portion
     that is over cutoff frequency */
  cut_freq(filter,del,n);
  

  /* Resort filter : before, filter had phase delay,*/
  /* i.e. ideal h[0] was at filter[len-1], now for fft-filtering */
  /* resort to fft-symmetry to get lin. phase */
 
  for (i=0;i<=n;i++)
    filter[i]=filter[i+n-1];
  
  for (i=n+1;i<2*n;i++)
    filter[i]=filter[2*n-i];
  

  return(filter);
}


/* This routine implements the ideal ramp filter but with 
   a cutoff frequency as specified on the commandline. 
   So
   H(omega) = H_ramp(omega) * rect(omega/omega_c) 

   where rect(x)=1 for |x|<1
   and H_ramp is the ideal highpass.
*/
double* ramp_Filter(int n,struct cmdline_params *cmdline,struct sino_info *sino)
{
  double *filter;
  int i;
  double del;

  /* allocate filter */
  filter = (double *)get_spc(2*n, sizeof(double));
  
  del = cmdline->freq_cutoff;
  printf("ramp filter cutoff is %f percent.\n",100.0*del); 
  
  /* Calculate impulse response in time domain, 
     hrl is ideal ramp filter response 1/(4*delta_t)*(2*delta(n)-sinc^2(n/2)),
   */
  for(i=-n+1;i<=n;i++)
    filter[i+n-1] = hrl(i,sino); 
  
  /* So far, filter response is only accurate for omega<=omega_c */
  /* convolve with sinc() to chop off frequency domain portion
     that is over cutoff frequency */
  cut_freq(filter,del,n);
  

  /* Resort filter : before, filter had phase delay,*/
  /* i.e. ideal h[0] was at filter[len-1], now for fft-filtering */
  /* resort to fft-symmetry to get lin. phase */
 
  for (i=0;i<=n;i++)
    filter[i]=filter[i+n-1];
  
  for (i=n+1;i<2*n;i++)
    filter[i]=filter[2*n-i];
  
  return(filter);

}




/* convolution , used in compute_filter to */
/* convolve Gaussian*HammW with ideal recon-filter*/

void conv(double *out, double *in1, int len1, double *in2, int len2)
{
  int i,n;

  for (n=0;n<len1+len2-1;n++)
    {
      out[n]=0.0;
      for (i=0;i<len1;i++)
	if ((n-i>=0) && (n-i<len2))
	  out[n] += (in1[i]*in2[n-i]);
    }
}



void normalize_filter_to_sum1(double *filt, int len)
{
  int i;
  double sum=0.0;
 
  for (i=0;i<len;i++)
    sum+=(filt[i]);
 
  for (i=0;i<len;i++)
    filt[i]/=sum;
}




/* hrl returns 1/(4*delta_t)*(   2*delta(m)- (sinc(m/2))^2)  */
/* this is the ideal reconstruction filter */

double hrl(int m, struct sino_info *sino)
{ 
    double temp;
    double ans;

    if (m == 0)
      ans= 1.0/(4.0*sino->delta_t);
    else
      {
        temp = PI_1*m/2;
        temp = sin(temp)/temp;
        temp =  - temp*temp;
	ans= temp/(4.0*sino->delta_t) ;
      }
    return(ans);
}

double dhrl(double m, struct sino_info *sino)
{ 
    double temp,temp2,omeg,ans;

    if (fabs(m)<1e-8)
      ans=1.0/(4.0*sino->delta_t);
    else
    {
        omeg = PI_1*m/2.0;
        temp = sin(omeg)/omeg;
        temp2 = 2*sin(omeg*2)/(omeg*2);
        temp = temp2 -temp*temp;
        ans=temp/(4.0*sino->delta_t) ;
    }
    return(ans);
}







void backproject(double **image,double **Y,struct sino_info *sino,struct geom_info *geom)
{
  double x=0.0,y=0.0;
  double theta,t,dat,findex;
  int i,j,k;
  double *cs,*sn; 
       /* arrays to precompute cos(theta_k), sin(theta_k) */
  
  double *im_pt; 
       /* pointer to image[i][j]  Assume image is */
       /* allocated as 1-D array w/ pointers on each row */

  printf("Backprojecting ... \n");


  /* Precompute sin and cos for all projection angles */
 
  cs=(double*)malloc(sizeof(double)*sino->n_theta);
  sn=(double*)malloc(sizeof(double)*sino->n_theta);
  theta=sino->theta_0;
  for (k=0;k<sino->n_theta;k++)
    {
     sn[k]=sin(theta);
     cs[k]=cos(theta);
     theta+=sino->delta_theta;
    }



  /* Start Backprojection */
  im_pt=&image[0][0];
  y=geom->y_0;
  for (i=0;i<geom->n_y;i++)
    {
      x=geom->x_0;
      for (j=0;j<geom->n_x;j++)
	{
	  for (k=0;k<sino->n_theta;k++)
	    {
	      t=x*cs[k]-y*sn[k];
	      findex=(t-sino->t_0)/sino->delta_t;
	      
	      /* Check whether location is out of */
              /* projection area for this angle */
	      /* If so,  set image[i][j] to zero, */
              /* break k loop and go to next image location */
	      if ((findex<0) || (findex > sino->n_t - 1 ))
		{
		  *im_pt=0.0;
		  break;
		}
	     
	      dat=interpolate(findex, Y, k); 
                  /* bi-linear interpolation between par. proj */
	      
	      *im_pt += dat;
	    }
	  
	  /* Clip to >= zero, normalize */
	  if (*im_pt <= 0.0)
	    *im_pt = 0.0;
	  else
	    *im_pt *= (PI_1/(double)sino->n_theta);

	  im_pt++;
	  x += geom->delta;
	}
      y += geom->delta;
    }
  
  free((void*)cs);
  free((void*)sn);
}



/* Interpolate is used in backproject to */
/* perform bi-linear interploation between parallel projections */

double interpolate(double findex, double **Y, int angle_index)
{
  int index,ce,fl;
  double out;
  
  /* check whether findex is integer */
  index=(int)findex;
  if ((findex-(double)index>-1E-10) && (findex-(double)index<1E-10))
    out=Y[angle_index][index];
  else /* if not integer, then bilinear interpolation between parallel projections */
    {
      ce=(int)(findex+1.0); /* ceiling */
      fl=(int)findex; /* floor  */

      out=(((double)ce)-findex)*Y[angle_index][fl] + (findex-(double)fl)*Y[angle_index][ce];
    }
 
  return(out);

}

void mean_value_correct(double **image, double pmass, struct geom_info *geom)
{
  double imass=0.0;
  double  cor_factor,mx;
  int    i,j;
  
  for (i=0;i<geom->n_y;i++)
    for (j=0;j<geom->n_x;j++)
      imass += (double)image[i][j];
  imass /= ((double)(geom->n_y*geom->n_x)); 
  
  cor_factor=(double)(pmass/imass);
  
  printf("pmass:%f imass:%f cor_factor:%f\n",pmass,imass,cor_factor);

  for (i=0;i<geom->n_y;i++)
    for (j=0;j<geom->n_x;j++)
      image[i][j] *= cor_factor;

  /* Calculate Mean just to printf */
  mx = 0.0;
  for (i=0;i<geom->n_y;i++)
    for (j=0;j<geom->n_x;j++)
      mx += image[i][j];

  mx /= (double)(geom->n_y*geom->n_x);

 
  printf("The mean of the CBP image is %f\n",mx);
  printf("The min,max of the CBP image are %.3f,%.3f\n",min1(image,geom->n_y,geom->n_x),max1(image,geom->n_y,geom->n_x));
} 









double calc_pmass(double **Y, struct sino_info *sino, struct geom_info *geom)
{
  double pmass;
  double meanY=0.0;
  int i,j;
  
  for (i=0;i<sino->n_theta;i++)
    for (j=0;j<sino->n_t;j++)
      meanY+=(double)Y[i][j];

  meanY=meanY/((double)(sino->n_theta*sino->n_t));

  pmass = meanY*((double)sino->n_t)*sino->delta_t/(geom->delta*geom->delta*(double)(geom->n_y*geom->n_x)); 

  return(pmass);
}













/*void init_sp_col(struct column *sp_col,struct sino_info *sino)
{
  sp_col->val = (double *) get_spc(sino->n_t*sino->n_theta, sizeof(double));
  sp_col->row = (int *) get_spc(sino->n_t*sino->n_theta, sizeof(int));
  sp_col->multiplier = 1.0; 
}
*/

void cmdline_error()
{
  fprintf(stderr,"Usage:\n");
  fprintf(stderr,"cbp [-p plane (default=0)] [-r recon_resolution] [-f freq-cutoff Hann (0.8)] cfg_file out_file\n\n");
  exit(-1);  
}



void parse_cmdline(struct cmdline_params *cmdline,int argc,char **argv)
{
  int i,ct;
  int stat_plane=0,stat_res=0;

  cmdline->plane=0;
  cmdline->single_frame=-1;
  cmdline->sglfr_th0cor_FLAG=0;
  cmdline->INTERP_FLAG=0;
  cmdline->SIGMA_FLAG=0;
  cmdline->no_iter_rec=-1;
  cmdline->resol=-1;
  cmdline->use_ramp_filter = 0;
  cmdline->freq_cutoff = 0.8;

  for (;;)
    { 
      i= getopt(argc, argv, "p:r:f:");
      if (i==EOF)
        break;
      else
        {
          if (i == '?')
            cmdline_error();
	  if (i=='p')
            {
	      cmdline->plane = atoi(optarg);
	      stat_plane=1;
            }
	  if (i=='f')
            {
	      cmdline->freq_cutoff = atof(optarg);
	    }

	  if (i=='r')
	    {
	      cmdline->resol=atoi(optarg);
	      stat_res=1;
	    }
	  /*if (i=='R')
	    cmdline->use_ramp_filter=1;*/
 
	  
	  
	}
    }
  ct=optind;
  if ((argc-optind < 2) || (argc-optind > 2))
    {
      fprintf(stderr,"\nError: Wrong number of arguments.\n\a\n");
      cmdline_error();
    }

  cmdline->cfg_fname = argv[ct];
  ct++;
  
  cmdline->output = argv[ct];
  ct++;

  if (!stat_res)
    printf("No resolution specified. Reconstructing at default resolution.\n");
  
}


void read_data(struct all_data *data,struct column *sp_col,struct cfg_info *cfg_info,int resol)
{
  MATFile *fin;  
  
#ifdef MATLAB5
  mxArray *pm;
#endif

  int Ang,T;
  int Ang2,T2;
  char *fname;
  int sfile_plane;
  

  data->sino.nrm_FLAG=0;

  /* if set to matlab, read input sinogram and nrm file from matfiles */
  if (cfg_info->MATLAB_FLAG)
    {
      fname=cfg_info->sino_fname[0];
     /* read input sinogram Y */
     fin = matOpen(fname, "r");
     if (fin == NULL)
       {
         fprintf(stderr,"Cannot read MatLab input file %s\n",fname);
         exit(-1);
       }
  
     data->Y = read_mat_double_image(&Ang,&T,"counts",fin);
     if (data->Y == NULL)
       {
         fprintf(stderr,"Cannot read counts matrix \n");
         exit(-1);
       }
     /*read_mat_sino_info(fin, &data->sino);*/
 
    
      pm = matGetArray(fin,"sep_d");
      if(pm != NULL) 
	data->sino.sep_d = mxGetScalar(pm);
      else
	{
	  fprintf(stderr,"Error reading file %s: No variable sep_d.\n\a\n",fname);
	  exit(-1);
	}
      mxDestroyArray(pm);
      
      pm = matGetArray(fin,"theta_0");
      if(pm != NULL) 
	data->sino.theta_0 = mxGetScalar(pm);
      else
	{
	  fprintf(stderr,"Error reading file %s: No variable theta_0.\n\a\n",fname);
	  exit(-1);
	}
      mxDestroyArray(pm);
      pm = matGetArray(fin,"no_frames");
      if(pm != NULL) 
	data->sino.no_frames = (int)(mxGetScalar(pm)+0.5);
      else
	{
	  fprintf(stderr,"Error reading file %s: No variable no_frames.\n\a\n",fname);
	  exit(-1);
	}
      mxDestroyArray(pm);
      if (56*data->sino.no_frames!=Ang)
	{
	  fprintf(stderr,"Error reading file %s: Row dimension of sinogram doesn't equal 56*no_frames.\n\a\n",fname);
	  exit(-1);
	}
      data->sino.n_t=112;
      data->sino.n_theta=56*data->sino.no_frames;
      
      matClose(fin);


      if (cfg_info->NRM_FLAG)
	{
	  data->sino.nrm_FLAG=1; 
	  fin = matOpen(cfg_info->nrm_file, "r");
	  if (fin == NULL)
	    {
	      fprintf(stderr,"Cannot read MatLab normalization file %s\n",cfg_info->nrm_file);
	      exit(-1);
	    }

	  data->sino.nrm_img = read_mat_double_image(&Ang2,&T2,"counts",fin);
	  if (data->sino.nrm_img == NULL) 
	    {
	      fprintf(stderr,"Error reading normalization file %s: No counts array.\n\a\n",cfg_info->nrm_file);
	      exit(-1);
	    }
  
	  if (T2!=data->sino.n_t)/* || (Ang2*data->sino.no_frames!=Ang))*/
	    {
	      fprintf(stderr,"Normalization sinogram size doesn't match.\n\a\n");
	      exit(-1);
	    }
	  matClose(fin);
	}
    
    }
  else
    {
      /* Use IDL routines to read input and normalization */
      data->Y=cti_read_sino(cfg_info->sino_fname, cfg_info->no_sino, cfg_info->plane, &data->sino); 
    
      if (cfg_info->NRM_FLAG)
        {
          data->sino.nrm_img=read_nrm_plane(cfg_info->nrm_file, cfg_info->plane,  cfg_info->deadtime_FLAG);
          data->sino.nrm_FLAG=1;
        }
      else
         data->sino.nrm_FLAG=0;
     }


 


  /* Read attenuation */
  if (cfg_info->attn_MODE == ATTN_FILE)
    {
      sfile_plane = 2*(int)(cfg_info->plane/9);
      if (9*(int)(cfg_info->plane/9) != cfg_info->plane)
	{
	  fprintf(stderr,"Error: Reading attenuation file for cross plane (%d) not implemented.\n\a\n",cfg_info->plane);
	  exit(-1);
	}
      data->sino.attn.sin_img=cti_read_attn_sin(cfg_info->attn_fname, sfile_plane, &data->sino.attn.img_rows, &data->sino.attn.img_cols);
      printf("N_ang=%d, M_offsets=%d\n",data->sino.attn.img_rows,data->sino.attn.img_cols);
     
    }
 

  
  
}
  

void write_output(struct cfg_info *cfg_info,struct all_data *data,double sigma,struct geom_info *geom)
{
  MATFile *fout;
  int N,M,i,j;
  double **Xi;
  char matrix[20];
  double **tmp;
  
  tmp=(double**)multialloc(sizeof(double),2,1,1);

  N=geom->n_y;
  M=geom->n_x;
  printf("Writing output image %s\n",cfg_info->output);
  
  /* open file */
  /*sprintf(fname,"%s_pl%d.mat",cmdline->output,cmdline->plane);*/
  fout = matOpen(cfg_info->output, "w");
  if (fout == NULL)
    {
      fprintf(stderr,"Cannot write MatLab file %s\n",cfg_info->output);
      exit(-1);
    }
  
  /* write sigma */
  
  tmp[0][0]=cfg_info->fov;
  write_mat_double_image(tmp,1,1,"fov",fout);
  
  tmp[0][0]=data->sino.sep_d;
  write_mat_double_image(tmp,1,1,"sep_d",fout);
  tmp[0][0]=data->sino.dthck;
  write_mat_double_image(tmp,1,1,"dthck",fout);
  /*tmp[0][0]=sigma;
    write_mat_double_image(tmp,1,1,"sigma",fout);*/
  
  sprintf(matrix,"img");
  if (!cfg_info->INTERP_FLAG)
    {
      write_mat_double_image(data->X,N,M,matrix,fout);
    }
  else
    {
      printf("Interpolating output by 2...");
      Xi=(double**)multialloc(sizeof(double),2,2*N,2*M);
      
      for (i=0;i<N;i++)
	for (j=0;j<M;j++)
	  {
	    Xi[2*i][2*j]=Xi[2*i+1][2*j]=Xi[2*i][2*j+1]=Xi[2*i+1][2*j+1]=data->X[i][j];
	  }
      
      write_mat_double_image(Xi,2*N,2*M,matrix,fout);

      multifree((void*)Xi,2);
    }

  
  matClose(fout);
  multifree((void*)tmp,2);

}
  

 
 
void free_memory(struct all_data *data, struct column *sp_col,struct cfg_info *cfg_info,double **Ycbp)
{
  printf("Freeing memory ...\n");
  
  free_img((void **)data->X);
  /*free_img((void **)data->mask);*/
  free_img((void **)data->Y);
  /*free((void *)sp_col->val);
    free((void *)sp_col->row);*/
  multifree((void**)Ycbp,2);
 
  
  if (cfg_info->NRM_FLAG)
    free_img((void **)data->sino.nrm_img);
 
  if (cfg_info->attn_MODE==ATTN_FILE)
    multifree((void *)data->sino.attn.sin_img,2);
 
  if (cfg_info->MATLAB_FLAG==0)
    {
      multifree((void*)data->sino.frames_starttime,2);
      multifree((void*)data->sino.frames_duration,2);
     
      multifree((void*)data->sino.dtime.cor_singles,3);
      multifree((void*)data->sino.dtime.uncor_singles,3);
     
      free((void*)data->sino.dtime.scan_start_second);
      free((void*)data->sino.dtime.scan_start_minute);
      free((void*)data->sino.dtime.scan_start_hour);
      free((void*)data->sino.dtime.scan_start_day);
    }
}
 








void set_parameters(struct cmdline_params *cmdline,struct cfg_info *cfg_info,struct all_data *data,struct geom_info *geom)
{
  char *suf;
  int resol;
  double field_of_view;


  /* copy necessary cmdline params into cfg_info */
  strcpy(cfg_info->output,cmdline->output);
  /*strcpy(cfg_info->initial,cmdline->initial);*/
  cfg_info->plane = cmdline->plane;
  cfg_info->INTERP_FLAG = cmdline->INTERP_FLAG;
  
  /* copy optional cmdline params that override defaults in cfg_file */
  if (cmdline->no_iter_rec!=-1)
    cfg_info->no_iter_rec = cmdline->no_iter_rec;
  if (cmdline->resol!=-1)
    cfg_info->resol = cmdline->resol;
  if (cmdline->SIGMA_FLAG)
    cfg_info->sigma = cmdline->sigma;

  
  /* check if input is matlab or cti-scn file */
  suf=strrchr(cfg_info->sino_fname[0],'.');
  if (!strcmp(suf,".mat"))
    cfg_info->MATLAB_FLAG=1;
  else if (!strcmp(suf,".scn"))
    cfg_info->MATLAB_FLAG=0;
  else
    {
      fprintf(stderr,"Error: Specified sinogram filename %s doesn't have .mat or .scn suffix.\n\a\n",cfg_info->sino_fname[0]);
      exit(-1);
    }
  if (cfg_info->MATLAB_FLAG==1)
    {
      if (cfg_info->plane != 0)
	{
	  fprintf(stderr,"Error: plane=%d: Cannot specify plane in matlab mode.\n\a\n",cfg_info->plane);
	  exit(-1);
	}
    }


  /* copy params into sino: This should be changed in future, 
     instead of keeping copy in sino, should pass cfg_info
     to subroutines. */
  data->sino.DECAY_FLAG= cfg_info->decay_FLAG;
  data->sino.prj_max_t = cfg_info->prj_max_t;
  data->sino.analyt_sysresp = cfg_info->analyt_FLAG;
  data->sino.no_planes=1;
  data->sino.plane=cfg_info->plane;
  data->sino.DEADTIME_FLAG = cfg_info->deadtime_FLAG;
  data->sino.dtime.no_sino_files = cfg_info->no_sino;
  
  /* Set attenuation parameters */
  data->sino.attn.attn_analyt_r = cfg_info->attn_circ_rad;
  data->sino.attn.attn_cnst = cfg_info->attn_cnst;
  data->sino.attn.ATTN_MODE = cfg_info->attn_MODE;

  data->scatter = cfg_info->scatter;
  /*printf("Setting scatter to %g.\n",data->scatter);*/

  data->sino.frame_duration=0;
  data->sino.frame_start_time=0;


  resol=cfg_info->resol;
  field_of_view=cfg_info->fov; /* in cm  */
  geom->field_of_view=field_of_view;
  geom->n_y = resol;
  geom->n_x = resol;
  geom->delta =field_of_view/(double)geom->n_x;
  geom->dia = field_of_view; /* ?? */
  geom->x_0 = geom->y_0 = -0.5*((double)resol-1.0)*geom->delta;
  



}


void change_parameters_for_sgl_frame_recon(struct all_data *data, struct cmdline_params *cmdline)
{
  double inc_theta;
  int i;

  printf("Setting parameters for single frame reconstruction\n");
  if ((cmdline->single_frame<0)||(cmdline->single_frame>=data->sino.no_frames))
    {
      fprintf(stderr,"Error: Specified frame %d out of range [0, %d].\n\a\n", cmdline->single_frame,data->sino.no_frames-1);
      exit(-1);
    }
  if (cmdline->sglfr_th0cor_FLAG)
    {
      inc_theta=PI_1/data->sino.no_frames;
      data->sino.theta_0=-inc_theta*(double)cmdline->single_frame;
      printf("Calculating theta0 for single frame %d as theta_0=%f\n",cmdline->single_frame,data->sino.theta_0);
    } 
  
  
  data->sino.no_frames=1;
  data->sino.n_theta=56;
  for (i=0;i<6272;i++)
    data->Y[0][i]=data->Y[0][i+cmdline->single_frame*6272];
  printf("\n");
}
 

void print_configuration(struct cfg_info *cfg_info, struct cmdline_params *cmdline, struct all_data *data)
{
  if (cmdline->single_frame==-1)
     printf("Reconstructing all frames\n");
   else
     {
       printf("Reconstructing SINGLE frame %d, ",cmdline->single_frame);
       if (cmdline->sglfr_th0cor_FLAG)
	 printf("using theta_0 for that frame.\n");
       else
	 printf("not correcting for theta_0.\n");
     }

  printf("Scatter=%f\n",data->scatter);
  
  printf("Sino-config:\n");
  printf("n_theta = %d\nn_t=%d\ntheta_0=%f\nsep_d=%f\n",data->sino.n_theta,data->sino.n_t,data->sino.theta_0,data->sino.sep_d);
  printf("no_frames=%d\nnrm_FLAG=%d\nhalflife=%f\nDECAY_FLAG=%d\nplane=%d\nATTN_MODE=%d\n",data->sino.no_frames,data->sino.nrm_FLAG,data->sino.halflife,data->sino.DECAY_FLAG,data->sino.plane,data->sino.attn.ATTN_MODE);
  printf("attn_analyt_r=%f\nattn_cnst=%f\nprj_max_t=%f\nanalyt_sysresp=%d\nno_planes=%d\n",data->sino.attn.attn_analyt_r,data->sino.attn.attn_cnst,data->sino.prj_max_t,data->sino.analyt_sysresp,data->sino.no_planes);
  
 
  
  if (cfg_info->MATLAB_FLAG)
    printf("No Decay modeling in matlab mode.\n");
  else
    {
      if (data->sino.DECAY_FLAG)
	printf("Decay Modeling ON\n");
      else
	printf("Decay Modeling OFF\n");
    }

  printf("\n\n");

}



void cut_freq(double *filter,double del,int n)
{
  double flt2[1024];
  int i,k;
  
  for(i=-n+1;i<=n;i++)
    {
      flt2[i+n-1]=0.0;
      for (k=-n+1;k<=n;k++)
	flt2[i+n-1] += (filter[k+n-1] * snc(del*(double)(i-k))*del);
    } 
  for(i=-n+1;i<=n;i++)
    filter[i+n-1] = flt2[i+n-1];
  
}

double snc(double n)
{
  double out,x;

  if (fabs(n)<1e-9)
    out=1.0;
  else
    {
      x=PI_1*n;
      out=sin(x) / x;
    }
  return(out);


}



double min1(double **img,int N, int M)
{
  int i,j;
  double min;
  
  min = img[0][0];
  for (i=0;i<N;i++)
    for(j=0;j<M;j++)
      if ( img[i][j] < min )
	min = img[i][j];
  
  return(min);
}


double max1(double **img,int N, int M)
{
  int i,j;
  double max;
  
  max = img[0][0];
  
  for (i=0;i<N;i++)
    for(j=0;j<M;j++)
      if (img[i][j] > max)
	max = img[i][j];
  
  return(max);
}
